/*
 * @(#)GraphicsTest.java   1.9 98/06/29
 *
 * Copyright (c) 1997, 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.*;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Scrollbar;
import java.awt.ScrollPane;
import java.awt.TextField;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.DecimalFormat;
import java.util.EventListener;
import java.util.StringTokenizer;

class MapPanel extends Panel implements AdjustmentListener,FocusListener,ItemListener,MouseMotionListener {


     TextField   textFieldRegId;
     TextField   textFieldRegSel;
     Choice      cProduct,cItem,cTheme;

     public int  curProduct=-1,curTheme=-1,curItem=-1,curGroup;
     public int  MapZoom = 1;

     Label  ProductChoiceLabel;
     Label  ItemChoiceLabel;
     Label  ThemeChoiceLabel;

     Panel  panelMemberStates = new Panel();

     ScrollPane myScrollPane;

     Scrollbar ScrollbarMin;
     Scrollbar ScrollbarMax;
     Scrollbar ScrollbarMid;

     public RegionMap    myRegionMap;
     public Map          myMapApplet;

     GiniDiagram  myGiniDiagram;
     Legend       myLegend;
     Barchart     myBarchart;
     MsText       myMsText;
     RegText      myRegText;

     static String regSelText[];
     static int    nRegSel;
     static String oldSelText   = new String("*");

     // Constructor
     MapPanel(Map Parent) {


        this.setFont(new Font("Helvetia",Font.PLAIN,10));

        myMapApplet = Parent;

        setLayout(new BorderLayout());

        /* ----------------- Center Side ------------------------------ */

        myScrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        myRegionMap = new RegionMap(this);
        myScrollPane.add(myRegionMap);
        myScrollPane.setBackground(Color.white);

        add("Center",myScrollPane);
        myRegionMap.addMouseMotionListener(this);

        /* ----------------- Right Side (East) ----------------------- */

        Panel GridP = new Panel();
        GridP.setLayout(new GridLayout(2,1));

        /* ----------------- Right Side top: Member state results ---- */

        GridP.add(myMsText = new MsText(this));

            /* ----------------- Right Side down: Still empty ---- */

        GridP.add(myBarchart = new Barchart(this));

        add("East",GridP);

        /* ----------------- Left Side (West)------------------------ */

            /* ----------------- Left Side top: Regional results ---- */


        GridP = new Panel();
        GridP.setLayout(new GridLayout(2,1));

        Panel p = new Panel();
        p.setLayout(new BorderLayout());

        p.add("North",textFieldRegId = new TextField(null,20));
        p.add("Center",myRegText = new RegText(this));
        GridP.add(p);

            /* ----------------- Left Side down: Gini diagramm ---- */

        p = new Panel();
        Panel p1 = new Panel();
        p1.setLayout(new BorderLayout());

        ScrollbarMax = new Scrollbar(Scrollbar.HORIZONTAL);
        ScrollbarMax.addAdjustmentListener(this);
        p1.add("North",ScrollbarMax);

        p1.add("Center",myGiniDiagram = new GiniDiagram(myRegionMap));

        ScrollbarMin = new Scrollbar(Scrollbar.HORIZONTAL);
        ScrollbarMin.addAdjustmentListener(this);
        p1.add("South",ScrollbarMin);

        ScrollbarMid = new Scrollbar(Scrollbar.VERTICAL);
        ScrollbarMid.addAdjustmentListener(this);
        p1.add("West",ScrollbarMid);

        p.add(p1);

        p.add(myLegend = new Legend(myRegionMap));

        GridP.add(p);

        add("West",GridP);

        /* ----------------- Top (North) ------------------------------ */

        p = new Panel();
        p.setLayout( new GridLayout(2,6));
        p.setBackground(Color.gray);

        Label NLabel = new Label("Theme");
        p.add(NLabel);
        ProductChoiceLabel = new Label();
        p.add(ProductChoiceLabel);
        ItemChoiceLabel = new Label();
        p.add(ItemChoiceLabel);
        ThemeChoiceLabel = new Label();
        p.add(ThemeChoiceLabel);
        NLabel = new Label("Countries");
        p.add(NLabel);
        NLabel = new Label("Map zoom");
        p.add(NLabel);

        // choice for product group
        Choice c = new Choice();
        c.setName("Product group");
        String[] Labels = myMapApplet.GroupStr;
        int n = Labels.length;
        for (int i=0;i < n; i++)
           c.addItem(Labels[i]);
        c.addItemListener(this);
        c.select(curGroup = myMapApplet.getPreset("StartFile",Labels,c));
        p.add(c);


        // get scenarios
        String scens = myMapApplet.getParameter("Scenarios");
        if ( scens == null )
        {
           // open data file (all scenarios in one file)
           myMapApplet.nScen  = 0;
           String fileName = Labels[curGroup]+".CSV";
           myMapApplet.readData(myMapApplet.openFile(fileName),fileName,0);
        }
        else
        {
            StringTokenizer StrTok = new StringTokenizer(scens," ");
            myMapApplet.nScen  = StrTok.countTokens();

            myMapApplet.ScenStr = new String[myMapApplet.nScen];

            for ( int iScen = 0; iScen < myMapApplet.nScen; iScen++)
            {
                myMapApplet.ScenStr[iScen] = new String(StrTok.nextToken());
                String fileName = Labels[curGroup]+"_"+myMapApplet.ScenStr[iScen]+".CSV";
                myMapApplet.readData(myMapApplet.openFile(fileName),fileName,iScen);
            }
        }



        // choice for product selection
        c = cProduct = new Choice();
        c.setName("Products");
        c.addItemListener(this);
        p.add(c);


        // choice for item selection
        c= cItem = new Choice();
        c.setName("Items");
        c.addItemListener(this);
        p.add(c);

        // choice for theme selection
        c = cTheme = new Choice();
        c.setName("Themes");
        c.addItemListener(this);
        p.add(c);

        // choice for country selection
        textFieldRegSel = new TextField("*");
        nRegSel = 1;
        regSelText    = new String[nRegSel];
        regSelText[0] = new String("*");

        textFieldRegSel.setBackground(Color.white);
        textFieldRegSel.addFocusListener(this);
        p.add(textFieldRegSel);

        // choice for magnification
        c = new Choice();
        c.setName("Zoom");
        c.addItemListener(this);
        p.add(c);

        c.addItem("100");
        c.addItem("200");
        c.addItem("300");
        c.addItem("400");
        c.addItem("500");

        add("North",p);
        setSize(600, 600);

        updateChoices();
    }

    public Dimension getPreferredSize() {
        return new Dimension(600, 600);
    }

    public void updateChoices()
    {
        String[] Labels;
        int i,n;
        Choice c;


        c = cItem;
        ItemChoiceLabel.setText(myMapApplet.strItemDes);
        c.removeAll();
        Labels = myMapApplet.ItemStr;


        n = Labels.length;
        for ( i=0;i < n; i++)
           c.addItem(Labels[i]);
        c.select(curItem = myMapApplet.getPreset(myMapApplet.strItemDes,Labels,c));

        c = cProduct;
        c.removeAll();
        ProductChoiceLabel.setText(myMapApplet.strProductDes);
        Labels = myMapApplet.ProductStr;
        n = Labels.length;
        for ( i=0;i < n; i++)
           c.addItem(Labels[i]);
        c.select(curProduct = myMapApplet.getPreset(myMapApplet.strProductDes,Labels,c));

        c = cTheme;
        c.removeAll();
        ThemeChoiceLabel.setText(myMapApplet.strThemeDes);
        Labels = myMapApplet.ThemeStr;
        n = Labels.length;
        for ( i=0;i < n; i++)
        {



           c.addItem(Labels[i]);
        }
        c.select(curTheme = myMapApplet.getPreset(myMapApplet.strThemeDes,Labels,c));

        myRegionMap.setData(curProduct,curItem,curTheme);
    }



    public void adjustmentValueChanged(AdjustmentEvent e)
    {
       // one of the scrollbars was operated, send new value to Map to adjust value range
       myRegionMap.setNewMinMax(ScrollbarMin.getValue(),ScrollbarMid.getValue(),ScrollbarMax.getValue(),true);
       drawEverything();
    }

    public void focusLost(FocusEvent e)
    {
        if ( e.getSource() == textFieldRegSel  )
        {
           String newSelText = ((TextField) (e.getSource())).getText();
           if ( !oldSelText.equals(newSelText) )
           {
              oldSelText = newSelText;

              StringTokenizer StrTok = new StringTokenizer(newSelText," ");
              nRegSel = StrTok.countTokens();
              regSelText = new String[nRegSel];
              for ( int iRegSel = 0; iRegSel < nRegSel; iRegSel++)
                 regSelText[iRegSel] = StrTok.nextToken();

              myRegionMap.setData(curProduct,curItem,curTheme);
              myRegionMap.repaint();
              drawEverything();
           }
         }
    }

    public void focusGained(FocusEvent e)
    {
    }

    public void mouseMoved(MouseEvent e)
    {
         myRegionMap.mouseMovedTo( e.getX(), e.getY());
    }

    public void mouseDragged(MouseEvent e)
    {
    }


    public void itemStateChanged(ItemEvent e)
    {

        if ( e.getStateChange() != ItemEvent.SELECTED ) return;

        String CurLabel = (String) e.getItem();
        String myParent = ((Choice) e.getSource()).getName();

        if ( myParent.equals("Products"))
           curProduct = ((Choice) e.getSource()).getSelectedIndex();
        else if ( myParent.equals("Items"))
           curItem    = ((Choice) e.getSource()).getSelectedIndex();
        else if ( myParent.equals("Themes"))
           curTheme   = ((Choice) e.getSource()).getSelectedIndex();
        else if ( myParent.equals("Zoom"))
           MapZoom   = Integer.valueOf(CurLabel).intValue()/100;

        if ( myParent.equals("Product group") )
        {


           if ( myMapApplet.nScen != 0)
           {
              for ( int iScen = 0; iScen < myMapApplet.nScen; iScen++)
              {
                   myMapApplet.readData(myMapApplet.openFile(CurLabel+"_"+myMapApplet.ScenStr[iScen]+".CSV"),CurLabel+"_"+myMapApplet.ScenStr[iScen]+".CSV",iScen);
              }
           }
           else
              myMapApplet.readData(myMapApplet.openFile(CurLabel+".CSV"),CurLabel+".CSV",0);

           updateChoices();
        }

        if  ( !myParent.equals("Zoom") )
        {
           myRegionMap.setData(curProduct,curItem,curTheme);
           setRegTextArea();
        }
        else
        {
           int w = myScrollPane.getBounds().width;
           int h = myScrollPane.getBounds().height;
           myRegionMap.setSize(w*MapZoom,h*MapZoom);
           myRegionMap.repaint();
           Graphics g = this.getGraphics();
           paintComponents(g);
           return;
        }

        drawEverything();
    }

    boolean regIsSelected(int iReg)
    {
         if ( regSelText[0].equals("*") ) return true;

         for ( int iRegSel = 0; iRegSel < nRegSel; iRegSel++)
             if ( myMapApplet.RegionIds[iReg].substring(0,regSelText[iRegSel].length() ).equals(regSelText[iRegSel]) )
               return true;

         return false;
    }

    void setRegTextArea()
    {
        if ( myRegionMap.curPol == -1 )
           textFieldRegId.setText(" ");
        else
        {
           int curPol = myRegionMap.curPol;
           int curReg = myRegionMap.curReg;

           String NewContent;
           if ( myMapApplet.RegionStr[curPol] == null )
              NewContent = new String(myMapApplet.RegionIds[curReg]);
           else
              NewContent = new String(myMapApplet.RegionStr[curPol]+"("+myMapApplet.PolygonIds[curPol]+")");
           textFieldRegId.setText(NewContent);
        }

        myRegText.repaint();
        repaint();
    }

    void drawEverything()
    {
        myGiniDiagram.repaint();
        myLegend.repaint();
        myMsText.repaint();
        myBarchart.repaint();
        myRegionMap.repaint();
        myScrollPane.repaint();

        Graphics g = this.getGraphics();
        paintComponents(g);
    }
}

public class Map extends Applet {

    MapPanel      myMapPanel;
    static Polygon[] polygonsOri = null;

    public int[]            prtPolyonToRegion  = null;
    public static String[]  GroupStr           = null;
    public static String[]  RegionIds          = null;
    public static String[]  RegionStr          = null;
    public static String[]  PolygonIds         = null;
    public static String[]  ProductStr         = null;
    public static String[]  ProductIds         = null;
    public static String[]  ItemStr            = null;
    public static String[]  ThemeStr           = null;
    public static String[]  ScenStr           = null;

    public static String    strItemDes         = null;
    public static String    strProductDes      = null;
    public static String    strThemeDes        = null;

    public static int[][]   compToWhat=null;

    public static int       RightSideWidth;

    public int maxItemStrLength=0;
    public int maxRegionStrLength=0;

    public int nReg,nPol,nAggReg,nGroup,
               nProduct,nItem,nTheme,nThemeOri;
    static int nScen;

    public int[]  indecesAggReg = null;


    static float[][][][] Data = null;

    public void init() {

        setLayout(new BorderLayout());

        // read coordinate file
        String SR = getParameter("RightSideWidth");

 //        System.err.println("SR = "+SR);

        if ( SR == null)
            RightSideWidth = 250;
        else
            RightSideWidth = Integer.parseInt(SR);

        // read coordinate file
        String fileName = getParameter("CoordFile");
        showStatus("Reading coordinates file " + fileName + " ...");
        readCoord(openFile(fileName));
        showStatus(" ");

        // get names of data file from htm-file
        String fileNames = getParameter("DataFiles");
        StringTokenizer StrTok = new StringTokenizer(fileNames," ");
        int nGroup = StrTok.countTokens();

        GroupStr = new String[nGroup];
        for (int iGroup=0; iGroup < nGroup; iGroup++)
            GroupStr[iGroup] = StrTok.nextToken();


        add("Center", myMapPanel = new MapPanel(this));
    }

    int getPreset(String DesString,String Labels[],Choice c)
    {
        // check if there is a preset in the htm file
        String selected = getParameter(DesString);
        if ( selected == null ) return 0;

        int pos = selected.lastIndexOf(";");
        if ( pos != -1 )
        {
           c.setEnabled(false);
           selected = selected.substring(0,pos);
        }


        int n = Labels.length;
        for (int i=0;i < n; i++)
           if (Labels[i].equals(selected) )
              return i;
        return 0;
    }

  /* open a file based on its name in the sandbox, returning a buffered reader to it
     the buffered reader must be closed after the file was read */

  public BufferedReader openFile(String FileName)
  {
            InputStream ins;
            URL CoordUrl=null;
            BufferedReader bufr=null;

            URL m_baseURL = getDocumentBase();

            try { CoordUrl = new URL(m_baseURL,FileName);}
            catch (MalformedURLException e) { System.err.println("Error in parameter CoordUrl, "+e);}

            try {
                URLConnection CoordUrlConnect = CoordUrl.openConnection();
                CoordUrlConnect.connect();
                ins  = CoordUrlConnect.getInputStream();
                bufr = new BufferedReader(new InputStreamReader(ins));
            }
            catch(Exception e) {System.err.println("Error opening file " + FileName + ",  " +e);}

            return bufr;
    }

  /* ---------------- Read the coordinate file ----------------------------------------

     The following structure of the file is assumed and NOT checked by the program:

     Comma separated format

     1st line: # of polygons
     2nd - 1+#of polygons line: regionIds + nPnt x (x,y)

     Polygons must be defined with origin left,top

     The # of points is determined implicitely by the # of tokens in each polygon line

  */

  public void readCoord(BufferedReader bufr)
  {
            if ( bufr == null ) return;

            String line = null;
            int iPol=0,iLine=0;

            try {
                while ( (line = bufr.readLine())!= null)
                {
                     if ( iLine == 0 )
                     {
                        StringTokenizer StrTok = new StringTokenizer(line," ");
                        nPol       = Float.valueOf(StrTok.nextToken()).intValue();

                        // System.err.println("nPol = "+nPol);

                        polygonsOri         = new Polygon[nPol];
                        PolygonIds          = new String[nPol];
                        RegionStr           = new String[nPol];
                        prtPolyonToRegion   = new int[nPol];
                     }
                     else
                     {
                        StringTokenizer StrTok = new StringTokenizer(line,",\"");
                        int nTok = StrTok.countTokens();

                        // look up region
                        PolygonIds[iPol] = StrTok.nextToken();
                        // look up region description string
                        RegionStr[iPol]  = StrTok.nextToken();


                        Polygon CurPolygon = new Polygon();

                        for (int iPnt= 0; iPnt < (nTok-2)/2; iPnt++)
                        {
                            int x = Integer.valueOf(StrTok.nextToken()).intValue();
                            int y = Integer.valueOf(StrTok.nextToken()).intValue();
                            CurPolygon.addPoint( x , y);
                        }

                        polygonsOri[iPol]  = CurPolygon;
                        iPol++;
                     }
                     iLine++;
                }
                bufr.close();
            }
            catch(Exception e) {System.err.println("Error reading coord file, "+e);}
    }

  /* ---------------- Read the data file ---------------------------------------------

     The following structure of the file is assumed and NOT checked by the program:

     Comma separated format

     1st line: # of regions, # of products, # of items, # of themes followed by:
               Label for product choice, item choice, theme choice


     2nd - 1+#of products : productId + product description (to be shown in the related choice)

     2+#of products       : all Item  descriptions

     3+#of products       : all Theme descriptions

     Afterwards: data for individiual regions and product
                 on line for each product, data are organised as nTheme*(Item1, ...,ItemN)


     The programs adds automatic additional "themes" consisting of all pairs of relative
     changes between the original themes in the data file. The relations are calculated "on the fly"
     to spare memory, see below in getDataValue

  */
  public void readData(BufferedReader bufr,String fileName,int iScen)
  {


            if ( bufr == null )
               return;

            String line = null;

            showStatus("Reading data file " + fileName + " ...");

            int iReg,iProduct,iTheme,iItem,iLine=1;
            try {
                while ( (line = bufr.readLine())!= null)
                {

                    StringTokenizer StrTok = new StringTokenizer(line,",\"");

                    // System.err.println("line = "+line);

                    // first line: read data labels
                    if ( iLine == 1 )
                    {
                       if ( nScen == 0 || iScen == 0)
                       {
                          nReg     = Float.valueOf(StrTok.nextToken()).intValue();
                          nProduct = Float.valueOf(StrTok.nextToken()).intValue();
                          nItem    = Float.valueOf(StrTok.nextToken()).intValue();

                          if ( nScen > 0 )
                          {
                             int test = Float.valueOf(StrTok.nextToken()).intValue();
                             if ( test != 1 )
                                System.err.println("More then one scenario in file "+fileName+" while trying to read from multiple data file");

                             nTheme       = nScen;
                          }
                          else
                          {
                             nTheme       = Float.valueOf(StrTok.nextToken()).intValue();
                          }

                          nThemeOri = nTheme;

                          StrTok.nextToken();
                          strProductDes = StrTok.nextToken();
                          strItemDes    = StrTok.nextToken();
                          strThemeDes   = StrTok.nextToken();

                          Data = new float[nReg][nProduct][nItem][nThemeOri];

                          for (int iPol=0; iPol < nPol; iPol++)
                             prtPolyonToRegion[iPol] = -1;

                          // introduce changes labels
                          int cmpTheme;
                          for(cmpTheme=0;cmpTheme<nThemeOri;cmpTheme++)
                             for(iTheme=cmpTheme+1;iTheme<nThemeOri;iTheme++)
                                nTheme++;

                          ProductStr     = new String[nProduct];
                          ProductIds     = new String[nProduct];
                          RegionIds      = new String[nReg];
                          ItemStr        = new String[nItem];
                          ThemeStr       = new String[nTheme];
                          compToWhat     = new int[nTheme][2];
                       }
                       else if ( iScen != 0 )
                       {
                            // check if information matches
                            int nReg1     = Float.valueOf(StrTok.nextToken()).intValue();
                            int nProduct1 = Float.valueOf(StrTok.nextToken()).intValue();
                            int nItem1    = Float.valueOf(StrTok.nextToken()).intValue();

                            int test = Float.valueOf(StrTok.nextToken()).intValue();
                            if ( test != 1 )
                                System.err.println("More then one scenario in file "+fileName+" while trying to read from multiple data file");

                            if ( nReg != nReg1 || nProduct1 != nProduct || nItem1 != nItem )
                                System.err.println("File "+fileName+" does not match first file read");
                       }

                    }
                    else if ( iLine == 2 && iScen == 0 ) // read region ids from current file
                    {

                              for( iReg = 0;iReg < nReg; iReg++)
                              {
                                 RegionIds[iReg] = new String(StrTok.nextToken());
                                 // System.err.println("iReg = "+RegionIds[iReg]);
                              }

                              for (int iPol=0; iPol < nPol; iPol++)
                              {
                                  // System.err.println("iPol = "+iPol);

                                  // first try with national data
                                 for( iReg = 0;iReg < nReg; iReg++)
                                    if ( RegionIds[iReg].substring(0,2).equals(PolygonIds[iPol].substring(0,2)) )
                                        prtPolyonToRegion[iPol] = iReg;

                                   // System.err.println("Ids = "+PolygonIds[iPol]+" iReg = "+prtPolyonToRegion[iPol]);

                                  // then try with NUTS I data
                                 for( iReg = 0;iReg < nReg; iReg++)
                                    if ( RegionIds[iReg].substring(0,3).equals(PolygonIds[iPol].substring(0,3)) )
                                          prtPolyonToRegion[iPol] = iReg;

                                  // then try with NUTS II data
                                 for( iReg = 0;iReg < nReg; iReg++)
                                      if ( RegionIds[iReg].substring(0,4).equals(PolygonIds[iPol].substring(0,4)) )
                                          prtPolyonToRegion[iPol] = iReg;

                                  // System.err.println("Ids = "+PolygonIds[iPol]+" iReg = "+prtPolyonToRegion[iPol]);
                              }
                    }
                    else  if ( iLine == 3 && iScen == 0) // read product ids + labels
                    {
                        for( iProduct = 0;iProduct < nProduct; iProduct++)
                        {
                           ProductIds[iProduct] = new String(StrTok.nextToken());
                           ProductStr[iProduct] = new String(StrTok.nextToken());
                        }
                    }
                    else  if ( iLine == 4 && iScen == 0) // read Item labels
                    {
                        maxItemStrLength=0;
                        for( iItem = 0;iItem < nItem; iItem++)
                        {
                          ItemStr[iItem] = new String(StrTok.nextToken());
                          maxItemStrLength = Math.max(maxItemStrLength,ItemStr[iItem].length());
                        }
                    }
                    else  if ( iLine == 5 ) // read Theme labels
                    {

                        if ( nScen == 0 )
                           for( iTheme = 0;iTheme < nThemeOri; iTheme++)
                              ThemeStr[iTheme] = new String(StrTok.nextToken());
                        else
                            ThemeStr[iScen]     = new String(StrTok.nextToken());

                       // introduce changes labels
                       if ( nScen == 0 || iScen == nScen-1)
                       {
                          int cmpTheme,curTheme=nThemeOri;
                          for(cmpTheme=0;cmpTheme<nThemeOri;cmpTheme++)
                             for(iTheme=cmpTheme+1;iTheme<nThemeOri;iTheme++)
                             {
                                ThemeStr[curTheme]   = new String(ThemeStr[iTheme]+"/"+ThemeStr[cmpTheme]);

                                compToWhat[curTheme][0] = iTheme;
                                compToWhat[curTheme][1] = cmpTheme;
                                curTheme++;
                             }
                       }

                    }
                    else  if ( iLine > 5 )// read data for one region and one product
                    {
                        String CurProduct = StrTok.nextToken();
                        for ( iProduct=0; iProduct < nProduct; iProduct++)
                            if ( CurProduct.equals(ProductIds[iProduct]) ) break;

                        String CurReg = StrTok.nextToken();
                        for ( iReg=0; iReg < nReg; iReg++)
                            if ( CurReg.equals(RegionIds[iReg])) break;

                        if ( iReg != nReg && iProduct != nProduct) // found
                           if ( nScen == 0 )
                             for ( iTheme = 0; iTheme < nThemeOri; iTheme++)
                             for ( iItem = 0;  iItem  < nItem;  iItem++)
                                 Data[iReg][iProduct][iItem][iTheme] = Float.valueOf(StrTok.nextToken()).floatValue();
                           else
                             for ( iItem = 0;  iItem  < nItem;  iItem++)
                                 Data[iReg][iProduct][iItem][iScen] = Float.valueOf(StrTok.nextToken()).floatValue();
                    }
                    iLine++;
                }
                bufr.close();
            }
            catch(Exception e) {System.err.println("Error reading data file, "+e);}


            nAggReg = 0;
            for (iReg=0; iReg < nReg; iReg++)
                if ( RegionIds[iReg].length() > 4)
                   if ( RegionIds[iReg].substring(2,5).equals("000") )
                       nAggReg++;

            int iAgg;
            indecesAggReg = new int[nAggReg];
            for (iReg=0, iAgg=0; iReg < nReg; iReg++)
                if ( RegionIds[iReg].length() > 4)
                if ( RegionIds[iReg].substring(2,5).equals("000") )
                     indecesAggReg[iAgg++] = iReg;

            showStatus(" ");
            return;
    }

    public float getDataValue(int iReg,int iPro,int iItem,int iTheme)
    {
         if ( iTheme < nThemeOri )
            return Data[iReg][iPro][iItem][iTheme];
         else
         {
            if ( Data[iReg][iPro][iItem][compToWhat[iTheme][1]] == 0 )
               return Float.POSITIVE_INFINITY;

            else
               return (Data[iReg][iPro][iItem][compToWhat[iTheme][0]]/Data[iReg][iPro][iItem][compToWhat[iTheme][1]]-1)*100;
         }
    }

    public void destroy() {
        remove(myMapPanel);
    }

    public static void main(String args[])
    {
        AppletFrame.startApplet("Map", "Map", args);
    }

    public String getAppletInfo() {
        return "CAPRI Project: Java based Mapping Applet\n by W.Britz, November 1999.";
    }

}   // end class GraphicsTest


/*  --------------------------------------------------------------------------

 Here we have the class drawing the map

    -------------------------------------------------------------------------- */
class RegionMap extends Panel
{
  // class variables
  Polygon[] polygonsOri;

  public int   nPol=0;
  static int   nPolOri;

  public int   curReg,curPol,iPolOld=-1;

  static int   wr,hr,xr,yr,xScale,yScale;

  static float scale;
  static int   iMin,iMid,iMax;
  static float xMinR,xMidR,xMaxR;


  public DecimalFormat DecFormat = new DecimalFormat();
  public int nDigits;

  Graphics ng;
  MapPanel myMapPanel;
  Map      myMapApplet;

  // current data values belonging to polygons
  public static float[]  xs = null;
  // pointer from polygons to Regions
  static int[]  prtPolyonToRegion  = null;

  // scaling for grey scale
  public float xmin,xmax;

  public RegionMap(MapPanel Parent)
  {
       myMapPanel = Parent;
       // get Map applet
       myMapApplet  = myMapPanel.myMapApplet;
       // get pointers from regions to polygons
       prtPolyonToRegion  = myMapApplet.prtPolyonToRegion;

       DecFormat.setGroupingSize(100);

       polygonsOri = myMapApplet.polygonsOri;
       nPolOri     = myMapApplet.nPol;

       // reserve memory for data belonging to polygons
       xs = new float[nPolOri];

       // default settings for cut off min/max in percent
       xMinR = (float) 0.05;
       xMidR = (float) 0.50;
       xMaxR = (float) 0.95;
  }

  public void setCountrySel(String newSelText)
  {
  }


  public String formatNumber(float x)
  {
       String numString;

       if (x == Float.POSITIVE_INFINITY)
           numString   = new String(" ");
       else
           numString   = new String(DecFormat.format(x));
       return numString;
  }

  // fill the polygon with new colours depending on new min/max bounds on data
  void setNewMinMax(int newMin,int newMid, int newMax,boolean sliderMoved)
  {
      // Polygon #
      iMin = newMin;
      iMid = nPol - newMid;
      iMax = newMax;

      if ( sliderMoved )
      {
         // related precentages (to be kept when the region selection is kept)
         xMinR = iMin/(float) (nPol-1);
         xMidR = iMid/(float) (nPol);
         xMaxR = iMax/(float) (nPol-1);
      }
  }

  public Color setColorForValue(float x)
  {

       if (x == Float.POSITIVE_INFINITY)
          return Color.lightGray;
       else if ( xs[iMax] <= xs[iMin] )
          return Color.white;

       Color NewColor;

       x = Math.min(Math.max(x,xs[iMin]),xs[iMax]);
       float xr = Math.min(xs[iMax],Math.max(xs[iMid],xs[iMin]));
       float xGray;

       if ( x <= xr && xr > xs[iMin] )  // below medium value: green
       {
          xGray = (float) (1.- Math.pow( (double) (xr-x)/(xr-xs[iMin]),0.7));
          NewColor = new Color(xGray,(float) Math.max(0.25,Math.pow( (double) xGray,0.3)),xGray);
       }
       else // above medium value: red
       {
          xGray = (float) (1. - Math.pow( (double) (x-xr)/(xs[iMax]-xr),0.7));
          NewColor = new Color((float) Math.max(0.40,Math.pow( (double) xGray,0.1)),xGray,xGray);
       }
       return NewColor;
  }

  // scale the polygon to the current size of panel
  void scalePolygon(Polygon PolCur,Polygon pOri)
  {
    for (int i = 0; i < pOri.npoints; ++i)
         PolCur.addPoint( (int) ((pOri.xpoints[i]-xr) * scale + xScale),
                          (int) ((pOri.ypoints[i]-yr) * scale + yScale));
  }


  // fill the polygon with new colours depending on new min/max bounds on data
  void fillPolygons()
  {

    // setting for xs[iMin] equals xs[iMax]
    int curReg;
    int curProduct = myMapPanel.curProduct;
    int curItem    = myMapPanel.curItem;
    int curTheme   = myMapPanel.curTheme;

    for (int iPol = 0; iPol < nPolOri; iPol++)
    {
       curReg = prtPolyonToRegion[iPol];

       if ( curReg != -1  )
       if ( myMapPanel.regIsSelected(curReg) )
       {
          Polygon PolCur = new Polygon();
          scalePolygon(PolCur,polygonsOri[iPol]);

          ng.setColor(setColorForValue(myMapApplet.getDataValue(curReg,curProduct,curItem,curTheme) ));
          ng.fillPolygon(PolCur);
          ng.setColor(Color.black);
          ng.drawPolygon(PolCur);
       }
    }
  }

  public void mouseMovedTo(int x, int y)
  {

       x = (int) ((x-xScale)/scale + xr);
       y = (int) ((y-yScale)/scale + yr);

       setCountrySel(myMapPanel.textFieldRegSel.getText());

       if ( iPolOld != -1)
          if ( polygonsOri[iPolOld].contains(x,y)) return;

       // assume outside of every region
       curPol = -1;
       iPolOld = -1;

       for ( int iPol= 0; iPol < nPolOri; iPol++)
           if ( polygonsOri[iPol].contains(x,y))
           {
              iPolOld = iPol;
              if ( prtPolyonToRegion[iPol] != -1 )
              {
                 curReg = prtPolyonToRegion[iPol];
                 curPol = iPol;
              }
              break;
           }

        myMapPanel.setRegTextArea();
    }


  // set data for regions
  void setData(int curProduct,int curItem,int curTheme)
  {
    int   iReg;

    if ( curTheme < myMapApplet.nThemeOri )
       nDigits = 0;
    else
       nDigits = 2;

    DecFormat.setMaximumFractionDigits(nDigits);

    int countPol=0;
    for ( int iPol=0; iPol < nPolOri; iPol++)
    {
        iReg = prtPolyonToRegion[iPol];
        if ( iReg != -1 )
        if ( myMapPanel.regIsSelected( iReg) )
        {
           if (countPol == 0 )
           {
               xr  = polygonsOri[iPol].getBounds().x;
               yr  = polygonsOri[iPol].getBounds().y;
               wr  = 0;
               hr  = 0;
           }
           xr  = Math.min(xr,polygonsOri[iPol].getBounds().x);
           yr  = Math.min(yr,polygonsOri[iPol].getBounds().y);
           wr  = Math.max(wr,polygonsOri[iPol].getBounds().x+polygonsOri[iPol].getBounds().width);
           hr  = Math.max(hr,polygonsOri[iPol].getBounds().y+polygonsOri[iPol].getBounds().height);
           xs[countPol++] = myMapApplet.getDataValue(iReg,curProduct,curItem,curTheme);
        }
    }
    // no region is selected: return and do nothing
    if ( countPol == 0 ) return;

    nPol=countPol;

    wr = wr - xr;
    hr = hr - yr;

    for ( int i=0; i < nPol; i++)
        if ( xs[i] == Float.POSITIVE_INFINITY) xs[i] = 0;

    // brutal sort of array + create pointer list from position in xs to polygon
    float xtmp;
    int  itmp;

    for ( int i=0; i < nPol; i++)
    {   // no change treated as a zero

       for ( int j=i+1; j < nPol; j++)
           if ( xs[j] < xs[i] )
           {
              xtmp = xs[i];
              xs[i]= xs[j];
              xs[j]= xtmp;
           }
     }

    // define format from current xmin and xmax

    xmax = xs[nPol-1];
    xmin = xs[0];

    iMin = (int) ((nPol-1)*xMinR);
    iMid = (int) ((nPol)*(1.-xMidR));
    iMax = (int) ((nPol-1)*xMaxR);

    myMapPanel.ScrollbarMin.setValues(iMin,1,0,nPol);
    myMapPanel.ScrollbarMid.setValues(iMid,1,1,nPol+1);
    myMapPanel.ScrollbarMax.setValues(iMax,1,0,nPol);
    setNewMinMax(iMin,iMid,iMax,false);
  }

  public Dimension getPreferredSize() {
     return new Dimension(600*myMapPanel.MapZoom,600*myMapPanel.MapZoom);
  }

  void fill(Graphics g, int x, int y, int w, int h)
  {
    ng = g.create();
    ng.translate(x,y);

    h -= h/40;
    w -= w/40;

    scale  = (float) (Math.min( ((float) w) /((float) wr),((float) h)/((float) hr)));

    // move x and y according to size of Panel
    xScale = x + (int) ((w - wr*scale)/2.);
    yScale = y + (int) ((h - hr*scale)/2.);

    fillPolygons();
 }

  public void paint(Graphics g) {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}

/*
   Draw a Gini Digramm (cummulative distribution )

   The y-axis spans the range between the map minimum and maximum
   The x-axis runs over the regions
*/


class GiniDiagram extends Panel
{

  RegionMap myRegionMap;

  public GiniDiagram(RegionMap Parent)
  {
     myRegionMap = Parent;
  }

  void fill(Graphics g, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.translate(x,y);

    int left   = (int) x + w/20;
    int top    = (int) y + h/20;

    w = w - 2*w/20;
    h = h - 2*h/20;

    float[] xs      = myRegionMap.xs;
    int     iMin    = myRegionMap.iMin;
    int     iMid    = myRegionMap.iMid;
    int     iMax    = myRegionMap.iMax;

    ng.setColor(Color.white);
    ng.fillRect(left,top,w,h);

    int curX,curY,oldY=0,oldX=left-1,ph=h-(2*h)/50;

    // loop over width of Gini Box
    for ( int iPol = 0; iPol < myRegionMap.nPol; iPol++)
    {
        // x-coordinate
        curX = left + (int) ((float) ( ((iPol+1)*w)/myRegionMap.nPol));

        // y-coordinate is data value
        curY = (int) (ph*(xs[iPol]-xs[iMin])/(xs[iMax]-xs[iMin]));

        // raise bottom and lower top
        curY = Math.max( Math.min(curY,h-h/50),h/50);
        if ( iPol == 0 ) oldY=curY;

        //draw black line
        ng.setColor(Color.black);
        // on top
        ng.drawLine(oldX+1,top+h-curY-1,curX,top+h-curY-1);
        // on left side
        ng.drawLine(oldX,  top+h-oldY-1,oldX,top+h-curY-1);

        if ( iPol==iMin )
              ng.drawLine(oldX,top,oldX,top+h);
        else if (iPol == iMid || iPol == iMax)
              ng.drawLine(curX,top,curX,top+h);

        if ( iPol == iMid )
        {
           String curNum = myRegionMap.formatNumber(xs[iMid]);
           FontMetrics   curFontM = ng.getFontMetrics();

           if ( iMid < myRegionMap.nPol/2)
              ng.drawString(curNum,
                            curX,
                            Math.max(top+h-curY-curFontM.getHeight(),top+curFontM.getHeight())) ;
           else
              ng.drawString(curNum,
                            curX-curFontM.stringWidth(curNum),
                            Math.max(top+h-curY-curFontM.getHeight(),top+curFontM.getHeight())) ;
        }

        ng.setColor(myRegionMap.setColorForValue(xs[iPol]));
        ng.fillRect(oldX+1,top+h-curY,curX-oldX,curY);

        oldY=curY;
        oldX=curX;
    }

    ng.setColor(Color.black);
    ng.drawRect(left-1,top-1,w+1,h+1);
  }

  public Dimension getPreferredSize() {
     return new Dimension(200,200);
  }

  public void paint(Graphics g)
  {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}

class Legend extends Panel
{

  static RegionMap myRegionMap;

  public Legend(RegionMap Parent)
  {
       myRegionMap = Parent;
  }

  void fill(Graphics g, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.translate(x,y);

    // Legend
    int left   = x;
    int top    = (int) y + h/20;
    h = h - 2*h/20;

    float[] xs      = myRegionMap.xs;
    int     iMin    = myRegionMap.iMin;
    int     iMax    = myRegionMap.iMax;

    int ph=h-(2*h)/50;
    float xDat;
    // draw legend
    for ( int iStep=0; iStep < 6; iStep++)
    {
         xDat = xs[iMin] + (iStep*(xs[iMax]-xs[iMin]))/5;
         ng.setColor(myRegionMap.setColorForValue(xDat));
         ng.drawString(myRegionMap.formatNumber(xDat),
                       left,top+h-(iStep*h)/5+2);
    }
  }

  public Dimension getPreferredSize() {
     return new Dimension(30,160);
  }

  public void paint(Graphics g) {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}

// colored bar chart of member state results
class Barchart extends Panel
{

  MapPanel  myMapPanel;
  RegionMap myRegionMap;
  Map       myMapApplet;

  public Barchart(MapPanel Parent)
  {
       myMapPanel  = Parent;
       myRegionMap = myMapPanel.myRegionMap;
       myMapApplet = myMapPanel.myMapApplet;
  }


  void fill(Graphics g, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.translate(x,y);

    int left   = (int) x + 4;
    int top    = (int) y + h/10;

    int ht = h - 2 * h/5;
    int wt = w - 8;
    ng.setColor(Color.white);
    ng.fillRect(left,top,wt,ht);
    ng.setColor(Color.black);
    ng.drawRect(left-1,top-1,wt+1,ht+1);

    // calculate width of each bar
    int barW = wt /( myMapApplet.nAggReg * (myMapApplet.nThemeOri+2) + 1);

    int xOffs = (wt - barW * (myMapApplet.nAggReg * (myMapApplet.nThemeOri+2))) /2;

    // get max and min of member states
    int iAgg,iReg;
    int barH,xP=x+xOffs;

    int iPro   = myMapPanel.curProduct;
    int iItem  = myMapPanel.curItem;
    int curTheme  = myMapPanel.curTheme;
    int iTheme;

    ng.setColor(Color.black);

    float xDat[][][][] = myMapApplet.Data;

    float xmin = xDat[myMapApplet.indecesAggReg[0]][iPro][iItem][0];
    float xmax = xDat[myMapApplet.indecesAggReg[0]][iPro][iItem][0];
    for ( iAgg   = 0; iAgg   < myMapApplet.nAggReg; iAgg++)
    {
        // determine member state
        iReg = myMapApplet.indecesAggReg[iAgg];
        // draw member states labels
        ng.drawString(myMapApplet.RegionIds[iReg].substring(0,2),
                            xP,top+ht+ht/5);
       for ( iTheme = 0; iTheme < myMapApplet.nThemeOri; iTheme++)
       {
            xmin = Math.min(xmin,xDat[iReg][iPro][iItem][iTheme]);
            xmax = Math.max(xmax,xDat[iReg][iPro][iItem][iTheme]);
            xP = xP + barW;
       }
       xP = xP + 2*barW;
    }

    // determine scaling of bar charts
    float xBarScale = (float) ( ht*0.9/ (xmax-xmin) );
    float xCurVal;


    xP = xP=x+xOffs;

    // draw bar charts
    for ( iAgg   = 0; iAgg   < myMapApplet.nAggReg; iAgg++)
    {
       for ( iTheme = 0; iTheme < myMapApplet.nThemeOri; iTheme++)
       {
            iReg    = myMapApplet.indecesAggReg[iAgg];
            xCurVal = myMapApplet.getDataValue(iReg,iPro,iItem,iTheme);

            // determine height for current bar chart
            barH = (int) ( (xCurVal-xmin) * xBarScale );

            // set color as for region with the same data value in the map
            if ( curTheme < myMapApplet.nThemeOri )
               ng.setColor(myRegionMap.setColorForValue(xCurVal));
            else
               ng.setColor(Color.blue);

            ng.fillRect(xP,top+ht-barH,barW,barH);

            // draw outer shape of bar in black
            ng.setColor(Color.black);
            ng.drawRect(xP,top+ht-barH,barW,barH);

            xP = xP + barW;
       }
       xP = xP + 2*barW;
    }

    FontMetrics   curFontM = ng.getFontMetrics();
    String myName = new String("W.Britz,CAPRI team Bonn");
    ng.drawString(myName,x+w-curFontM.stringWidth(myName)-2,y+h-2);
  }

  public Dimension getPreferredSize() {
     return new Dimension(myMapApplet.RightSideWidth,160);
  }

  public void paint(Graphics g) {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}

// Member State Text
class MsText extends Panel
{

  MapPanel  myMapPanel;
  RegionMap myRegionMap;
  Map       myMapApplet;

  public MsText(MapPanel Parent)
  {
       myMapPanel  = Parent;
       myRegionMap = myMapPanel.myRegionMap;
       myMapApplet = myMapPanel.myMapApplet;
  }

  void fill(Graphics g, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.translate(x,y);

    int left   = (int) x + 4;
    int top    = (int) y + 4;

    int wt = w - 8;
    int ht = h - 8;
    ng.setColor(Color.white);
    ng.fillRect(left,top,wt,ht-1);

    ng.setColor(Color.black);
    ng.drawRect(left-1,top-1,wt+1,ht);

    int curProduct  = myMapPanel.curProduct;
    int curItem     = myMapPanel.curItem;
    int iReg,iAgg,iTheme;

    // draw Texts For Scenarios
    FontMetrics   curFontM = ng.getFontMetrics();

    int Lineh   = curFontM.getHeight() + curFontM.getHeight()/10;

    int curY  = top + 2 * Lineh;
    int curX  = left+1;

    String curString = null;
    // draw Texts Regional aggregates
    for (iAgg=0; iAgg < myMapApplet.nAggReg; iAgg++)
    {
       iReg = myMapApplet.indecesAggReg[iAgg];
       curString = myMapApplet.RegionIds[iReg].substring(0,2);

       ng.drawString(curString,left+1,curY);
       curY += Lineh;
       curX  = Math.max(curX,left+1+curFontM.stringWidth(curString));
    }

    int Inc  = (wt - curX)/myMapApplet.nThemeOri;

    myRegionMap.DecFormat.setMaximumFractionDigits(0);
    for (iTheme=0; iTheme < myMapApplet.nThemeOri; iTheme++)
    {
        curX += Inc;
        // draw heading
        curY = top + 2 * Lineh;
        curString = myMapApplet.ThemeStr[iTheme];
        ng.drawString(curString,curX-curFontM.stringWidth(curString),top+Lineh);

        // numbers
        for (iAgg=0; iAgg < myMapApplet.nAggReg; iAgg++)
        {
           iReg = myMapApplet.indecesAggReg[iAgg];
           curString = myRegionMap.formatNumber(myMapApplet.getDataValue(iReg,curProduct,curItem,iTheme));
           ng.drawString(curString,curX-curFontM.stringWidth(curString),curY);
           curY += Lineh;
        }
    }
    myRegionMap.DecFormat.setMaximumFractionDigits(myRegionMap.nDigits);
  }

  public Dimension getPreferredSize() {
     return new Dimension(myMapApplet.RightSideWidth,160);
  }

  public void paint(Graphics g) {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}

// Member State Text
class RegText extends Panel
{

  MapPanel  myMapPanel;
  RegionMap myRegionMap;
  Map       myMapApplet;

  public RegText(MapPanel Parent)
  {
       myMapPanel  = Parent;
       myRegionMap = myMapPanel.myRegionMap;
       myMapApplet = myMapPanel.myMapApplet;
  }

  void fill(Graphics g, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.translate(x,y);

    int left   = (int) x + 4;
    int top    = (int) y + 4;

    int wt = w - 8;
    int ht = h - 8;
    ng.setColor(Color.white);
    ng.fillRect(left,top,wt,ht-1);

    ng.setColor(Color.black);
    ng.drawRect(left-1,top-1,wt+1,ht);

    if ( myRegionMap.curPol == -1 ) return;

    int curReg      = myRegionMap.curReg;
    int curProduct  = myMapPanel.curProduct;
    int curTheme    = myMapPanel.curTheme;
    int curItem     = myMapPanel.curItem;
    if ( curTheme < myMapApplet.nThemeOri )
              myRegionMap.DecFormat.setMaximumFractionDigits(0);
    int iItem;

    // draw Texts For Items
    FontMetrics   curFontM = ng.getFontMetrics();

    int Lineh   = curFontM.getHeight() + curFontM.getHeight()/10;

    int curY   = top  + Lineh;
    int textX  = left + 1;

    int Inc,iBas=0,iCur=0;
    if ( curTheme >= myMapApplet.nThemeOri )
    {
       Inc  = (wt-textX)/3;
       iBas = myMapApplet.compToWhat[curTheme][1];
       iCur = myMapApplet.compToWhat[curTheme][0];
    }

    String curString = null;
    // draw Item texts
    for (iItem=0;iItem < myMapApplet.nItem; iItem++)
    {
       curString = myMapApplet.ItemStr[iItem];

       if ( iItem == curItem )
          ng.setColor(Color.blue);
       else
         ng.setColor(Color.black);

       if ( (curTheme < myMapApplet.nThemeOri) && (myMapApplet.getDataValue(curReg,curProduct,iItem,curTheme) != 0))
       {
         ng.drawString(curString,left+1,curY);
         curY += Lineh;
         textX  = Math.max(textX,left+1+curFontM.stringWidth(curString));
        }
        else if ( (myMapApplet.getDataValue(curReg,curProduct,iItem,iCur) != 0 ) || (myMapApplet.getDataValue(curReg,curProduct,iItem,iBas) != 0) )
        {
         ng.drawString(curString,left+1,curY);
         curY += Lineh;
         textX  = Math.max(textX,left+1+curFontM.stringWidth(curString));
        }
    }


    myRegionMap.DecFormat.setMaximumFractionDigits(0);
    if ( curTheme < myMapApplet.nThemeOri )
       Inc  = wt - textX;
    else
    {
       Inc  = (wt-textX)/3;
       iBas = myMapApplet.compToWhat[curTheme][1];
       iCur = myMapApplet.compToWhat[curTheme][0];
       textX = wt - Inc*3;
    }

    int curX;
    curY  = top  + Lineh;
    for (iItem=0; iItem < myMapApplet.nItem; iItem++)
    {

        if ( iItem == curItem )
           ng.setColor(Color.blue);
        else
           ng.setColor(Color.black);

        curX  = textX + Inc;

        if ( (curTheme < myMapApplet.nThemeOri) && (myMapApplet.getDataValue(curReg,curProduct,iItem,curTheme) != 0))
        {
           curString = myRegionMap.formatNumber(myMapApplet.getDataValue(curReg,curProduct,iItem,curTheme));
           ng.drawString(curString,curX-curFontM.stringWidth(curString),curY);
           curY += Lineh;
        }
        else if ( (myMapApplet.getDataValue(curReg,curProduct,iItem,iCur) != 0 ) || (myMapApplet.getDataValue(curReg,curProduct,iItem,iBas) != 0) )
        {
           myRegionMap.DecFormat.setMaximumFractionDigits(2);
           curString = myRegionMap.formatNumber(myMapApplet.getDataValue(curReg,curProduct,iItem,curTheme)) + "% (";
           ng.drawString(curString,curX-curFontM.stringWidth(curString),curY);
           myRegionMap.DecFormat.setMaximumFractionDigits(0);

           curString = myRegionMap.formatNumber(myMapApplet.getDataValue(curReg,curProduct,iItem,iCur)) +"/";
           curX += Inc;
           ng.drawString(curString,curX-curFontM.stringWidth(curString),curY);

           curString = myRegionMap.formatNumber(myMapApplet.getDataValue(curReg,curProduct,iItem,iBas)) +")";
           curX += Inc;
           ng.drawString(curString,curX-curFontM.stringWidth(curString),curY);
           curY += Lineh;
        }
    }

    myRegionMap.DecFormat.setMaximumFractionDigits(0);
  }

  public Dimension getPreferredSize() {
     return new Dimension(250,160);
  }

  public void paint(Graphics g) {
      Rectangle bounds = getBounds();
      fill(g, 0, 0,bounds.width ,bounds.height);
  }
}
